package org.rayflood.mikuvat.io;

import static org.rayflood.mikuvat.io.VSQDecoder.WIN31J;
import static org.rayflood.mikuvat.io.VSQFile.COMMON;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiEvent;

public class VSQTrack extends ImmutableObject{
	private static final long serialVersionUID = 6296614810415852578L;
	private String trackName;
	private CommonSection common;
	private MixerSetting mixer;
	private TimeLine timeLine;

	public VSQTrack(MidiEvent trackName, MixerSetting mixer, Map<String, List<String>> sections){
		this();
		setTrack(trackName, mixer, sections);
	}

	public VSQTrack(String trackName, MixerSetting mixer, Map<String, List<String>> sections){
		this();
		setTrack(trackName, mixer, sections);
	}

	public VSQTrack(Map<String, List<String>> sections){
		this();
		setTrack(sections);
	}

	public void setTrack(MidiEvent trackName, MixerSetting mixer, Map<String, List<String>> sections){
		setMixer(mixer);
		setTrack(sections);
		setTrackName(trackName);
	}

	public void setTrack(String trackName, MixerSetting mixer, Map<String, List<String>> sections){
		setMixer(mixer);
		setTrack(sections);
		setTrackName(trackName);
	}

	public void setTrack(Map<String, List<String>> sections){
		setCommon(new CommonSection(sections.get(COMMON)));
		setTimeLine(new TimeLine(sections));
	}

	public String toString(){
		List<String> sl = new ArrayList<String>();
		if(hasTrackName()){
			sl.add("TrackName=" + getTrackName());
		}
		if(hasCommon()){
			sl.add("Common={" + getCommon().toString() + "}");
		}
		if(hasMixer()){
			sl.add("Mixer={" + getMixer().toString() + "}");
		}
		if(hasTimeLine()){
			sl.add("TimeLine={" + getTimeLine().toString() + "}");
		}

		StringBuffer sb = new StringBuffer();
		sb.append("VSQTrack={");
		if(!sl.isEmpty()){
			sb.append(sl.get(0));
			for(int i = 1; i < sl.size(); i++){
				sb.append("," + sl.get(i));
			}
		}
		sb.append("}");
		return sb.toString();
	}

	public boolean isReadied(){
		boolean ready = true;
		ready &= hasTrackName();
		ready &= hasCommon() && getCommon().isReadied();
		ready &= hasMixer() && getMixer().isReadied();
		ready &= hasTimeLine() && getTimeLine().isReadied();
		return ready;
	}

	public int compareTo(ImmutableObject obj){
		VSQTrack vsqTrack = (VSQTrack)obj;
		return compare(this.getTrackName(), vsqTrack.getTrackName());
	}

	public VSQTrack over(Object obj, boolean fill, boolean has){
		VSQTrack vsqTrack = ((VSQTrack)obj).clone();
		if(vsqTrack.hasCommon()){
			CommonSection common = hasCommon() ? getCommon() : new CommonSection();
			setCommon(common.over(vsqTrack.getCommon(), fill, has));
		}
		if(vsqTrack.hasMixer()){
			MixerSetting mixer = hasMixer() ? getMixer() : new MixerSetting();
			setMixer(mixer.over(vsqTrack.getMixer(), fill, has));
		}
		if(vsqTrack.hasTimeLine()){
			TimeLine timeLine = hasTimeLine() ? getTimeLine() : new TimeLine();
			setTimeLine(timeLine.over(vsqTrack.getTimeLine(), fill, has));
		}
		if(vsqTrack.hasTrackName() && (fill || has == hasTrackName())){
			setTrackName(vsqTrack.getTrackName());
		}
		return this;
	}

	public VSQTrack setDefault(){
		setCommon(CommonSection.getDefault());
		setMixer(MixerSetting.getDefault());
		setTimeLine(TimeLine.getDefault());
		return this;
	}

	public static VSQTrack getDefault(){
		VSQTrack vsqTrack = new VSQTrack();
		vsqTrack.setDefault();
		return vsqTrack;
	}

	public VSQTrack delete(){
		deleteTrackName();
		deleteCommon();
		deleteMixer();
		deleteTimeLine();
		return this;
	}

	public boolean isDeleted(){
		boolean has = false;
		has |= hasTrackName();
		has |= hasCommon();
		has |= hasMixer();
		has |= hasTimeLine();
		return !has;
	}

	public VSQTrack(){
		delete();
	}

	public String getTrackName(){
		return trackName;
	}

	public MidiEvent getTrackNameME() throws Exception{
		byte mb[] = getTrackName().getBytes(WIN31J);
		MetaMessage mm = new MetaMessage();
		mm.setMessage(1, mb, mb.length);
		return new MidiEvent(mm, 0);
	}

	public void setTrackName(String trackName){
		this.trackName = trackName;
	}

	public void setTrackName(MidiEvent trackName){
		MetaMessage mm = (MetaMessage)trackName.getMessage();
		setTrackName(new String(mm.getData(), WIN31J));
	}

	public void deleteTrackName(){
		trackName = null;
	}

	public boolean hasTrackName(){
		return trackName != null;
	}

	public CommonSection getCommon(){
		return common;
	}

	public void setCommon(CommonSection common){
		if(common == null || common.isDeleted()){
			deleteCommon();
			return;
		}
		this.common = common;
		setTrackName(common.getName());
	}

	public void deleteCommon(){
		common = null;
	}

	public boolean hasCommon(){
		return common != null;
	}

	public MixerSetting getMixer(){
		return mixer;
	}

	public void setMixer(MixerSetting mixer){
		if(mixer == null || mixer.isDeleted()){
			deleteMixer();
			return;
		}
		this.mixer = mixer;
	}

	public void deleteMixer(){
		mixer = null;
	}

	public boolean hasMixer(){
		return mixer != null;
	}

	public TimeLine getTimeLine(){
		return timeLine;
	}

	public void setTimeLine(TimeLine timeLine){
		if(timeLine == null || timeLine.isDeleted()){
			deleteTimeLine();
			return;
		}
		this.timeLine = timeLine;
	}

	public void deleteTimeLine(){
		timeLine = null;
	}

	public boolean hasTimeLine(){
		return timeLine != null;
	}

	public VSQTrack clone(){
		return (VSQTrack)super.clone();
	}

	public static VSQTrack load(String file) throws Exception{
		return (VSQTrack)ImmutableObject.load(file);
	}

	public static VSQTrack load(File file) throws Exception{
		return (VSQTrack)ImmutableObject.load(file);
	}

	public void store(String file) throws Exception{
		super.store(file);
	}

	public void store(File file) throws Exception{
		super.store(file);
	}

	public static VSQTrack unzip(String file) throws Exception{
		return (VSQTrack)ImmutableObject.unzip(file);
	}

	public static VSQTrack unzip(File file) throws Exception{
		return (VSQTrack)ImmutableObject.unzip(file);
	}

	public void zip(String file) throws Exception{
		super.zip(file);
	}

	public void zip(File file) throws Exception{
		super.zip(file);
	}
}
